
object IterationTest {

  def main (argv:Array[String]):Unit = run(argv)

  // a simple iterator that takes its input from another iterator
  // (that delivers strings) and delivers the length of those strings

  def stringLengthIterator(it:Iterator[String]):Iterator[Int] = 
    new StringLengthIterator(it)

  private
  class StringLengthIterator (it:Iterator[String]) extends Iterator[Int] {
    
    // note: hasNext is a method that takes *no* arguments
    def hasNext:Boolean = 
      it.hasNext

    def next ():Int = it.next().size
  }



  def run (a:Array[String]) = {

    println("\n\n\n\n\n")
           
    println("Command-line arguments iteration = ")
    printN(a.iterator,10)
    println("Command-list arguments length iteration = ")
    printN(stringLengthIterator(a.iterator),10)
    println("")


    val b:Array[String] = Array("this","is","a","long","array","with","a",
				"lot","of","rather","boring","entries",
				"but","I","had","to","make","sure","that",
				"it","was","long","enough","to","exhibit",
				"the","behavior","I","wanted","to","exhibit")
    val bIter:Iterator[String] = b.iterator
    println("The full long array = " + b.mkString("[",",","]"))
    println("Long array iterator = ")
    printN(bIter,10)
    println("Long array length iterator (continued) = ")
    printN(stringLengthIterator(bIter),10)
    println("Long array iterator (continued) = ")
    printN(bIter,10)
    println("Long array length iterator (continued) = ")
    printN(stringLengthIterator(bIter),10)
    println("")


    val T = BinTree
    val T1:BinTree[String] = T.node("the",
				    T.node("quick",
					   T.node("brown",T.empty(),T.empty()),
					   T.empty()),
				    T.node("fox",
					   T.node("jumps",T.empty(),T.empty()),
					   T.node("over",
						  T.node("the",
							 T.node("lazy",
								T.empty(),
								T.empty()),
							 T.node("dog",
								T.empty(),
								T.empty())),
						  T.empty())))
    println("T1 = " + T1)
    println("Elements T1 (as a stream) = ")
    T1.print()
    println("Elements T1 (as a stream, paired with itself) = ")
    T1.zip(T1).print()
    val iter = T1.iterator
    println("T1 iterator = ")
    printN(iter,3)
    println("T1 length iterator (continued) = ")
    printN(stringLengthIterator(iter),3)
    println("T1 iterator (continued) = ")
    printN(iter,3)
    println("")

    // an iterator delivering infinitely many pieces of data
    println("Creating an iterator from the infinite stream of squares")
    val sqIter:Iterator[Int] = Stream.intsFrom(1).map((x:Int)=> x*x).iterator
    println("First 10 elements of the iterator = ")
    printN(sqIter,10)
    println("Next 10 elements of the iterator = ")
    printN(sqIter,10)
    println("Next  10 elements of the iterator = ")
    printN(sqIter,10)
    println("")

    // Question: can you create an iterator->stream adapter, that
    // converts an iterator into a stream?
    // Make sure the resulting stream actually behaves like a stream
  }


  def printN[T] (it:Iterator[T],n:Int):Unit = 
    if (it.hasNext) {
      if (n>0) {
	print(" [" + it.next() + "]");
	printN(it,n-1)
      } else 
	println(" ...")
    } else 
      println("")

}
